/*
 * Gimbal
 * Copyright (C) 2024 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

package net.moonleay.gimbal.client.keybindings

import net.minecraft.client.option.KeyBinding
import net.minecraft.client.util.InputUtil
import net.moonleay.gimbal.build.BuildConstants
import net.moonleay.gimbal.client.keybindings.impl.editormode.EnableInsertModeShortcut
import net.moonleay.gimbal.client.keybindings.impl.editormode.EnableReplaceModeShortcut
import net.moonleay.gimbal.client.keybindings.impl.editormodemodifier.ToggleBulldozerModifierShortcut
import net.moonleay.gimbal.client.keybindings.impl.editormodemodifier.ToggleForcePlaceModifierShortcut
import net.moonleay.gimbal.client.keybindings.impl.editormodemodifier.ToggleNoClipModifierShortcut
import net.moonleay.gimbal.client.keybindings.impl.editormodemodifier.ToggleNoUpdatesModifierShortcut
import net.moonleay.gimbal.client.keybindings.impl.gamemode.CreativeModeShortcut
import net.moonleay.gimbal.client.keybindings.impl.gamemode.SpectatorModeShortcut
import net.moonleay.gimbal.client.keybindings.impl.gamemode.SurvivalModeShortcut
import net.moonleay.gimbal.constants.TranslationKeys
import org.apache.logging.log4j.LogManager
import org.lwjgl.glfw.GLFW

object KeybindingRegistrar {

    private val LOGGER = LogManager.getLogger(BuildConstants.modName)

    fun registerKeybindings() {
        LOGGER.info("Registering keybindings...")
        registerSetGameModeKeybindings()
        registerSetEditorModeKeybindings()
        registerSetEditorModeModifierKeybindings()
        LOGGER.info("Keybindings have been registered.")
    }

    private fun registerSetEditorModeModifierKeybindings() {
        val toggleBulldozerModifierShortcut = KeyBinding(
            TranslationKeys.Keybindings.Binding.Editor.Modifier.BULLDOZER,
            InputUtil.Type.KEYSYM,
            GLFW.GLFW_KEY_UNKNOWN,
            TranslationKeys.Keybindings.Category.Editor.MODIFIER
        )
        val toggleForcePlaceModifierShortcut = KeyBinding(
            TranslationKeys.Keybindings.Binding.Editor.Modifier.FORCE_PLACE,
            InputUtil.Type.KEYSYM,
            GLFW.GLFW_KEY_UNKNOWN,
            TranslationKeys.Keybindings.Category.Editor.MODIFIER
        )
        val toggleNoUpdatesModifierShortcut = KeyBinding(
            TranslationKeys.Keybindings.Binding.Editor.Modifier.NO_UPDATES,
            InputUtil.Type.KEYSYM,
            GLFW.GLFW_KEY_UNKNOWN,
            TranslationKeys.Keybindings.Category.Editor.MODIFIER
        )
        val toggleNoClipModifierShortcut = KeyBinding(
            TranslationKeys.Keybindings.Binding.Editor.Modifier.NO_CLIP,
            InputUtil.Type.KEYSYM,
            GLFW.GLFW_KEY_UNKNOWN,
            TranslationKeys.Keybindings.Category.Editor.MODIFIER
        )

        KeybindingManager.registerShortcut(ToggleBulldozerModifierShortcut(toggleBulldozerModifierShortcut))
        KeybindingManager.registerShortcut(ToggleForcePlaceModifierShortcut(toggleForcePlaceModifierShortcut))
        KeybindingManager.registerShortcut(ToggleNoUpdatesModifierShortcut(toggleNoUpdatesModifierShortcut))
        KeybindingManager.registerShortcut(ToggleNoClipModifierShortcut(toggleNoClipModifierShortcut))
    }

    private fun registerSetEditorModeKeybindings() {
        val insertKeyBinding = KeyBinding(
            TranslationKeys.Keybindings.Binding.Editor.Mode.INSERT,
            InputUtil.Type.KEYSYM,
            GLFW.GLFW_KEY_I,
            TranslationKeys.Keybindings.Category.Editor.MODE
        )
        val replaceKeyBinding = KeyBinding(
            TranslationKeys.Keybindings.Binding.Editor.Mode.REPLACE,
            InputUtil.Type.KEYSYM,
            GLFW.GLFW_KEY_R,
            TranslationKeys.Keybindings.Category.Editor.MODE
        )
//        val visualKeyBinding = KeyBinding(
//            "gimbal.key.editor.mode.visual",
//            InputUtil.Type.KEYSYM,
//            GLFW.GLFW_KEY_V,
//            "gimbal.category.editormode"
//        )
        KeybindingManager.registerShortcut(EnableInsertModeShortcut(insertKeyBinding))
        KeybindingManager.registerShortcut(EnableReplaceModeShortcut(replaceKeyBinding))
//        KeybindingManager.registerShortcut(EnableVisualModeShortcut(visualKeyBinding))
    }

    private fun registerSetGameModeKeybindings() {
        val survivalKeyBinding = KeyBinding(
            TranslationKeys.Keybindings.Binding.Game.Mode.SURVIVAL,
            InputUtil.Type.KEYSYM,
            GLFW.GLFW_KEY_UNKNOWN,
            TranslationKeys.Keybindings.Category.Game.GAMEMODE
        )
        val creativeKeyBinding = KeyBinding(
            TranslationKeys.Keybindings.Binding.Game.Mode.CREATIVE,
            InputUtil.Type.KEYSYM,
            GLFW.GLFW_KEY_UNKNOWN,
            TranslationKeys.Keybindings.Category.Game.GAMEMODE
        )
        val spectatorKeyBinding = KeyBinding(
            TranslationKeys.Keybindings.Binding.Game.Mode.SPECTATOR,
            InputUtil.Type.KEYSYM,
            GLFW.GLFW_KEY_UNKNOWN,
            TranslationKeys.Keybindings.Category.Game.GAMEMODE
        )
        KeybindingManager.registerShortcut(SurvivalModeShortcut(survivalKeyBinding))
        KeybindingManager.registerShortcut(CreativeModeShortcut(creativeKeyBinding))
        KeybindingManager.registerShortcut(SpectatorModeShortcut(spectatorKeyBinding))
    }
}
